﻿
namespace Orchard.OData.Services
{
    using Microsoft.OData.Core.UriParser.Semantic;
    using Microsoft.OData.Core.UriParser.TreeNodeKinds;
    using Microsoft.OData.Core.UriParser.Visitors;
    using Microsoft.OData.Edm;
    using NHibernate;

    internal abstract class NHibernateAbstractVisitor<T> : QueryNodeVisitor<T> {
        protected readonly ICriteria contentItemVersionRecordCriteria;
        protected readonly ICriteria contentItemRecordCriteria;
        protected readonly IEdmModel edmModel;

        protected NHibernateAbstractVisitor(ICriteria contentItemVersionRecordCriteria, IEdmModel edmModel)
            : base() {
            this.contentItemVersionRecordCriteria = contentItemVersionRecordCriteria;
            this.contentItemRecordCriteria = contentItemVersionRecordCriteria.BindCriteriaByPath("ContentItemRecord");
            this.edmModel = edmModel;
        }

        public bool IsLogicalOperator(BinaryOperatorKind nodeKind) {
            return
                nodeKind == BinaryOperatorKind.Or ||
                nodeKind == BinaryOperatorKind.And;
        }

        public bool IsComparisonOperator(BinaryOperatorKind nodeKind) {
            return
                nodeKind == BinaryOperatorKind.Equal ||
                nodeKind == BinaryOperatorKind.GreaterThan ||
                nodeKind == BinaryOperatorKind.GreaterThanOrEqual ||
                nodeKind == BinaryOperatorKind.LessThan ||
                nodeKind == BinaryOperatorKind.LessThanOrEqual ||
                nodeKind == BinaryOperatorKind.NotEqual;
        }

        public bool IsArithmeticalOperator(BinaryOperatorKind nodeKind) {
            return
                nodeKind == BinaryOperatorKind.Add ||
                nodeKind == BinaryOperatorKind.Divide ||
                nodeKind == BinaryOperatorKind.Modulo ||
                nodeKind == BinaryOperatorKind.Multiply ||
                nodeKind == BinaryOperatorKind.Subtract;
        }

        public bool IsNullNode(QueryNode singlevalueNode) {
            return
                singlevalueNode.Kind == QueryNodeKind.None ||
                (singlevalueNode is ConstantNode && (singlevalueNode as ConstantNode).Value == null);
        }
    }
}